<?php
/*
 * Checksum library for CodeIgniter
 *
 * @author		Peder Fjällström
 * @license		https://fedoraproject.org/wiki/Licensing/Beerware
 * @link		http://earthpeople.se/labs/2012/04/signed-requests-with-codeigniter
 * @since		Version 0.1.0

 * USAGE:
 * 1. place this file in application/libraries/
 * 2. load or autoload it
 * 3. in your controller, use this to validate a request
 *    if($this->checksum->validate()){
 * 		echo "good checksum!";
 *    }
 * 4. call your controller with ?checksum= by passing checksum as a post variable.
 * 5. the value of checksum must be calculated correctly on the client side (ios app/flash/etc). bad requests are logged in application/logs, together with the correct checksum for the request.
 * 
 * checksum: sha1('yourcontroller/yourmethod/?fieldstring|salt);
 * 
 * "fieldstring" is all the post or get variabled, formatted like name=value&name2=value2&etc
 * post and get variables must be sorted on key name (using same algorithm as php ksort).
 * 
 * "salt" is the codeigniter encryption key, set in application/config/config.php
 * 
 */

class checksum{
	
	function __construct()
	{
		$this->CI =& get_instance();
		$this->salt = $this->CI->config->config['encryption_key'];
		$this->request_checksum = '';
	}
	
	function validate()
	{
		$calculated_checksum = $this->calculate();
		if($calculated_checksum === $this->request_checksum){
			return true;
		}else{
			return false;
		}
	}
	
	function calculate(){
		$fieldstring = '';
		if($this->CI->input->server('REQUEST_METHOD') === 'POST'){
			$fields = $this->CI->input->post();
		}else if($this->CI->input->server('REQUEST_METHOD') === 'GET'){
			$fields = $this->CI->input->get();
		}
		if(!empty($fields['checksum'])){
			$this->request_checksum = $fields['checksum'];
			unset($fields['checksum']);
		}else{
			log_message('DEBUG', 'empty checksum!');
			return false;
		}
		if(is_array($fields)){
			ksort($fields);
			$fieldstring = http_build_query($fields);
		}
		$calculated_checksum = sha1($this->CI->uri->uri_string.'/?'.$fieldstring.'|'.$this->salt);
		log_message('INFO', 'checksum for this request would be: '.$calculated_checksum);
		return $calculated_checksum;
	}
	
}